from .template import Template


class Documents():
    """
    Documents - это одна таблица со всеми документами.
    Вообще в концепции предполагалось, что таких таблиц должно быть много под каждый тип для удобства поиска. 
    То есть отдельно таблица с паспортами, отдельно с правами, отдельно с какими-нибудь разрешениями и так далее. 
    Что пока непонятно - документов может быть много разных.
    """

    # ID METASERVICE
    id_metaservice = Template.id_metaservice

    # DATA
    check_date = models.DateTimeField(
        auto_now_add=True,
        help_text="timestamp проверки"
    )
    check_level = models.IntegerField(
        null=True,
        blank=True,
        help_text="информация об уровне проверки. Документ может быть проверен как платформой, так и мета-сервисом, так и партнером мета-сервиса, а может быть и никем (просто загружен). Указывается, так как достоверность проверки разная. Документ, проверенный только на низком уровне, не принимается во внимание как имеющийся до прохождения более высокоуровневой проверки. Информацию об уровнях проверки можно посмотреть по словарю Requirements. В данной таблице хранится информация о наиболее высоком уровне проверки."
    )
    expire_date = models.DateField(
        null=True,
        blank=True,
        help_text="срок окончания действия документа."
    )
    status = models.CharField(
        max_length=150,
        default="deactivated",
        help_text="активность на основе системы заявок"
    )
    data_path = models.CharField(
        max_length=1024,
        help_text="Путь к документу"
    )
    doc_type = models.CharField(
        max_length=2,
        choices=DOC_TYPES,
        help_text="тип документа (паспорт/паспорт 1 страница и т д) в соответствии с классификатором типов документов (см описание в Requirements)"
    )

    # OTHER
    is_global = Template.is_global
    is_visible = Template.is_visible

    # FK
    company_id = models.ForeignKey(
        Company,
        on_delete=models.DO_NOTHING,
        null=True,
        help_text="идентификатор компании, к которой относится документ, если таковая есть (может не быть)"
    )
    user_id = models.ForeignKey(
        get_user_model(),
        related_name="user_id_doc",
        on_delete=models.DO_NOTHING,
        help_text="уникальный идентификатор пользователя (конкретного клиентского аккаунта) являющегося владельцем данного документа"
    )
    ticket_status = models.ForeignKey(
        Ticket,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        help_text="id последнего актуального тикета, касающийся статуса. Если он меняет статус на закрытый - вызывается проверка, которая смотрит, нет ли другого открытого по пользователю."
    )
    checked_by = models.ForeignKey(
        get_user_model(),
        null=True,
        blank=True,
        related_name="checked_by_doc",
        on_delete=models.DO_NOTHING,
        help_text="userid проверившего"
    )